@@ -1,5 +1,5 @@
 ##
-## $Release: 0.7.0 $
+## $Release: 0.8.1 $
 ## copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
 ##
 ## Permission is hereby granted, free of charge, to any person obtaining
@@ -29,7 +29,7 @@
    http://www.kuwata-lab.com/tenjin/pytenjin-examples.html
 """
 
-__release__  = "0.7.0"
+__release__  = "0.8.1"
 __license__  = "MIT License"
 __all__      = ['Template', 'Engine', 'helpers', 'html', ]
 
@@ -280,16 +280,27 @@ def _create_html_module():
         return _escape_pattern.sub(_escape_callable, s)
         #return s.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;').replace('"','&quot;')
 
-    def tagattr(klass=None, **kwargs):
+    def tagattr(name, expr, value=None, escape=True):
+        """(experimental) Return ' name="value"' if expr is true value, else '' (empty string).
+           If value is not specified, expr is used as value instead."""
+        if not expr: return ''
+        if value is None: value = expr
+        if escape: value = escape_xml(to_str(value))
+        return ' %s="%s"' % (name, value)
+
+    def tagattrs(**kwargs):
         """(experimental) built html tag attribtes.
            ex.
-           >>> tagattr(src='img.png', size=20)
-           ' src="img.png" size="20"'
-           >>> tagattr(src='', size=0)
+           >>> tagattrs(klass='main', size=20)
+           ' class="main" size="20"'
+           >>> tagattrs(klass='', size=0)
            ''
         """
-        if klass: kwargs['class'] = klass
-        return ' '.join(['%s="%s"' % (k, escape_xml(to_str(v))) for k, v in kwargs.items() if v])
+        if 'klass' in kwargs: kwargs['class'] = kwargs.pop('klass')
+        if 'checked'  in kwargs: kwargs['checked']  = kwargs.pop('checked')  and 'checked'  or None
+        if 'selected' in kwargs: kwargs['selected'] = kwargs.pop('selected') and 'selected' or None
+        if 'disabled' in kwargs: kwargs['disabled'] = kwargs.pop('disabled') and 'disabled' or None
+        return ''.join([' %s="%s"' % (k, escape_xml(to_str(v))) for k, v in kwargs.items() if v])
 
     def checked(expr):
         """return ' checked="checked"' if expr is true."""
@@ -315,7 +326,7 @@ def _create_html_module():
             return ''
         return nl2br(escape_xml(text).replace('  ', ' &nbsp;'))
 
-    def nv(name, value, sep=None, klass=None, checked=None, disabled=None, **kwargs):
+    def nv(name, value, sep=None, **kwargs):
         """(experimental) Build name and value attributes.
            ex.
            >>> nv('rank', 'A')
@@ -329,22 +340,39 @@ def _create_html_module():
         """
         s = sep and 'name="%s" value="%s" id="%s"' % (name, value, name+sep+value) \
                 or  'name="%s" value="%s"'         % (name, escape_xml(value))
-        if klass:    kwargs['class'] = klass
-        if checked:  kwargs['checked'] = 'checked'
-        if disabled: kwargs['disabled'] = 'disabled'
-        return kwargs and s + ' ' + tagattr(**kwargs) or s
+        return kwargs and s + tagattrs(**kwargs) or s
+
+    def new_cycle(*values):
+        """Generate cycle object.
+           ex.
+             cycle = new_cycle('odd', 'even')
+             print(cycle())   #=> 'odd'
+             print(cycle())   #=> 'even'
+             print(cycle())   #=> 'odd'
+             print(cycle())   #=> 'even'
+        """
+        def gen(values):
+            n = len(values)
+            i = 0
+            while True:
+                yield values[i]
+                i = (i + 1) % n
+        if   python2:  return gen(values).next
+        elif python3:  return gen(values).__next__
 
     mod = _create_module('tenjin.helpers.html')
     mod._escape_table = _escape_table
     mod.escape_xml = escape_xml
     mod.escape     = escape_xml
     mod.tagattr    = tagattr
+    mod.tagattrs   = tagattrs
     mod.checked    = checked
     mod.selected   = selected
     mod.disabled   = disabled
     mod.nl2br      = nl2br
     mod.text2html  = text2html
     mod.nv         = nv
+    mod.new_cycle  = new_cycle
     return mod
 
 helpers.html = _create_html_module()
@@ -433,6 +461,7 @@ class Template(object):
                 self.newline = "\r\n"
             else:
                 self.newline = "\n"
+        self._stmt_not_added_yet = True
 
     def before_convert(self, buf):
         #buf.append('_buf = []; ')
@@ -650,6 +679,11 @@ class Template(object):
             buf.extend((self.escapefunc, "(", self.tostrfunc, "(", code, ")), "))
 
     def add_stmt(self, buf, code):
+        if self._stmt_not_added_yet:
+            # insert dummy if-stmt between buf[-2] and buf[-1]
+            if buf and buf[-1] != "\n" and buf[-1].isspace():
+                buf[-1:-1] = ("if True: ## dummy\n", )
+            self._stmt_not_added_yet = False
         if self.newline == "\r\n":
             code = code.replace("\r\n", "\n")
         buf.append(code)
@@ -772,7 +806,7 @@ class CacheStorage(object):
 
     def __init__(self, postfix='.cache'):
         self.postfix = postfix
-        self.items = {}
+        self.items = {}    # key: full path, value: template object
 
     def get(self, fullpath, create_template):
         """get template object. if not found, load attributes from cache file and restore  template object."""
@@ -798,7 +832,7 @@ class CacheStorage(object):
 
     def unset(self, fullpath):
         """remove template object from dict and cache file."""
-        self.items.delete(self)
+        self.items.pop(fullpath, None)
         return self._delete(fullpath)
 
     def clear(self):
@@ -840,7 +874,7 @@ class FileCacheStorage(CacheStorage):
 
     def _delete(self, fullpath):
         cachepath = self._cachename(fullpath)
-        if os.path.isfile(cachepath): os.path.unlink(cachepath)
+        if os.path.isfile(cachepath): os.unlink(cachepath)
 
 
 class MarshalCacheStorage(FileCacheStorage):
@@ -1091,19 +1125,21 @@ class Engine(object):
         assert filename and fullpath
         cache = self.cache
         template = cache and cache.get(fullpath, self.templateclass) or None
+        mtime = None
         if template:
             assert template.timestamp is not None
-            if template.timestamp < os.path.getmtime(filename):
+            mtime = os.path.getmtime(filename)
+            if template.timestamp != mtime:
                 #if cache: cache.delete(path)
                 template = None
                 if logger: logger.info("[tenjin.Engine] cache is old (filename=%s, template=%s)" % (repr(filename), repr(template)))
         if not template:
-            curr_time = time.time()
+            if not mtime: mtime = os.path.getmtime(filename)
             if self.preprocess:   ## required for preprocess
                 if _context is None: _context = {}
                 if _globals is None: _globals = sys._getframe(1).f_globals
             template = self._create_template(filename, _context, _globals)
-            template.timestamp = curr_time
+            template.timestamp = mtime
             if cache:
                 if not template.bytecode: template.compile()
                 cache.set(fullpath, template)
