@@ -1,5 +1,5 @@
 ###
-### $Release: 0.8.0 $
+### $Release: 0.8.1 $
 ### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
 ###
 
@@ -363,6 +363,28 @@ class EngineTest(unittest.TestCase, TestCaseHelper):
         finally:
             _remove_files(filenames.values())
 
+
+    def test_cachefile_unset(self):
+        try:
+            input = "<?py x = 10 ?>"
+            template_name = "cachefile_delete.pyhtml"
+            open(template_name, 'w').write(input)
+            storage = tenjin.MarshalCacheStorage()
+            engine = tenjin.Engine(cache=storage)
+            engine.render(template_name)
+            fullpath = os.path.abspath(template_name)
+            self.assertTrue(fullpath in storage.items)
+            self.assertExists(template_name + '.cache')
+            storage.unset(fullpath)
+            self.assertFalse(fullpath in storage.items)
+            self.assertNotExist(template_name + '.cache')
+            def f():
+                storage.unset(fullpath)
+            self.assertNotRaise(f)
+        finally:
+            _remove_files([template_name, template_name+'.cache'])
+
+
     def test_change_layout(self):
         data = EngineTest.testdata['test_change_layout']
         ## setup
